import matplotlib.pyplot as plt
import numpy as np

def graph(file, dic, losses, temp_IGBT, temp_diode, temp_case):
    """
    This function generates and displays two graphs based on the given file, losses, and dictionary.
    The first graph shows Torque and Speed as a function of Time.
    The second graph shows Current and Total losses as a function of Time.

    Parameters:
    file (pandas DataFrame): A pandas DataFrame representation of the Excel file
    losses (Losses object): An instance of the Losses class containing calculated losses
    dic (dict): Dictionary of constants and file parameters
    """

    # Extract relevant data from the Excel file
    time = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_time"]-1)].values
    speed = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_speed"]-1)].values
    torque = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_torque"]-1)].values
    i_ref = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_current"]-1)].values
    total_losses = losses.total_losses

    # Plot Torque and Speed vs Time
    fig1, ax1 = plt.subplots()
    ax1.plot(time, torque, label='Torque')
    ax1.set_xlabel('Time')
    ax1.set_ylabel('Torque')
    ax1.set_title('Torque & Speed vs Time')
    ax1.legend()

    ax2 = ax1.twinx()
    ax2.plot(time, speed, color='red', label='Speed')
    ax2.set_ylabel('Speed')
    ax2.legend()

    # Plot Current and Total losses vs Time
    fig2, ax3 = plt.subplots()
    ax3.plot(time, i_ref, label='Current')
    ax3.set_xlabel('Time')
    ax3.set_ylabel('Current')
    ax3.set_title('Current vs Time')
    ax3.legend()

    fig2b, ax4 = plt.subplots()
    ax4.plot(time, total_losses, color='red', label='Total losses')
    ax4.set_xlabel('Time')
    ax4.set_ylabel('Total losses vs Time')
    ax4.legend()

    # Plot Temperatures vs Time
    fig3, ax5 = plt.subplots()
    ax5.plot(time, temp_IGBT, label='Temp_IGBT')
    ax5.plot(time, temp_diode, label='Temp_diode')
    ax5.set_xlabel('Time')
    ax5.set_ylabel('Temperature')
    ax5.set_title('Temperatures vs Time')
    ax5.legend()
  
    # Créer les données de D et n/Nf
    D = np.linspace(0, 1, 100)  # Valeurs de D de 0 à 1
    n_Nf = D  # n/Nf est égal à D dans ce cas
    
    # Tracer la courbe
    fig4, ax6 = plt.subplots()
    ax6.plot(n_Nf, D)
    ax6.set_xlabel('n/Nf')
    ax6.set_ylabel('D')
    ax6.set_title('Courbe linéaire de D en fonction de n/Nf')

    # Display the plots
    plt.show()



def calculate_damage_cumulation(counter_IGBT, counter_Nf_IGBT, counter_diode, counter_Nf_diode, losses, dic, file):
    """
    This function calculates a series of damage-related quantities based on the given inputs.

    Parameters:
    counter_IGBT (numpy array): Array of counter values for the IGBT
    counter_Nf_IGBT (numpy array): Array of counter_Nf values for the IGBT
    counter_diode (numpy array): Array of counter values for the diode
    counter_Nf_diode (numpy array): Array of counter_Nf values for the diode
    losses (Losses object): An instance of the Losses class containing calculated losses
    dic (dict): Dictionary of constants and file parameters
    file (pandas DataFrame): A pandas DataFrame representation of the Excel file

    Returns:
    Lifetime_IGBT (float): Calculated lifetime for the IGBT
    Lifetime_diode (float): Calculated lifetime for the diode
    number_of_km_IGBT (float): Calculated kilometers for IGBT
    E_kwh (float): Calculated energy in kWh
    E_kwh_byhours (float): Calculated energy per hour in kWh
    rendemment (float): Calculated efficiency
    """
  
    # Extract time and power from the Excel file
    time = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_time"]-1)].values
    power = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_power"]-1)].values

    # Get the cycle time from the dictionary
    time_cycle = dic["time_cycle"]
  
    # Calculate the damage for the IGBT and diode
    d_IGBT = counter_IGBT[2, 0] / counter_Nf_IGBT[2, 0]
    for i in range(1, len(counter_Nf_IGBT[0])):
        d_IGBT += counter_IGBT[2, i] / counter_Nf_IGBT[2, i]
    Lifetime_IGBT = 1 / (d_IGBT)

    d_diode = counter_diode[2, 0] / counter_Nf_diode[2, 0]
    for i in range(1, len(counter_Nf_diode[0])):
        d_diode += counter_diode[2, i] / counter_Nf_diode[2, i]
    Lifetime_diode = 1 / (d_diode)
  
    # Calculate additional quantities based on the calculated damage
    d = min(d_IGBT, d_diode)
    number_of_km_IGBT = 1 / d * 23.266
    e_kwh = (sum(losses.total_losses) / 3600000) * (time_cycle / len(time))
    e_kwh_byhours = e_kwh * 2
    efficiency = 1 - e_kwh / (sum(power) / 3600000 * (time_cycle / len(time)))

    return Lifetime_IGBT, Lifetime_diode, number_of_km_IGBT, e_kwh_byhours, efficiency



from numpy import convolve
from powderul.function.materials import Materials
from powderul.function.poly_somme import poly_somme

class ZthMaterials:
  """
  Class representing the thermal impedance of materials for IGBT and diode. It uses Foster's model.
  """

  def __init__(self, dic):
    """
    Initializes a ZthMaterials instance. 

    Parameters:
    dic (dict): A dictionary containing thermal resistance and thermal capacitance for IGBT and diode.
    """

    # Extracting values from the dictionary
    self.layers_Rth_IGBT_jc = dic["layers_Rth_jc_IGBT"]
    self.layers_Cth_IGBT_jc = dic["layers_Cth_jc_IGBT"]
    self.layers_Rth_diode_jc = dic["layers_Rth_jc_diode"]
    self.layers_Cth_diode_jc = dic["layers_Cth_jc_diode"]
    self.Rth_ch = dic["Rth_ch"]
    self.Cth_ch = dic["Cth_ch"]
    self.Rth_hf = dic["Rth_hf"]

    self.Num_materials_IGBT = len(self.layers_Rth_IGBT_jc)
    self.Num_materials_diode = len(self.layers_Rth_diode_jc)

    # Initializing remaining properties
    self.num_foster_IGBT = self.den_foster_IGBT = None
    self.num_foster_diode = self.den_foster_diode = None
    self.num_ch = self.den_ch = None
    self.num_hf = self.den_hf = None
    self.num_cf = self.den_cf = None
    self.materials_IGBT = []
    self.materials_diode = []

    # Creating instances of the Materials class
    for i in range(self.Num_materials_IGBT):
      material = Materials(self.layers_Rth_IGBT_jc[i],
                           self.layers_Cth_IGBT_jc[i])
      self.materials_IGBT.append(material)

    for i in range(self.Num_materials_diode):
      material = Materials(self.layers_Rth_diode_jc[i],
                           self.layers_Cth_diode_jc[i])
      self.materials_diode.append(material)


  def calculate_zth_foster(self):
    """
    Calculates thermal impedance using the Foster model.
    """

    num = {}
    den = {}

    # Calculating for IGBT
    for i in range(self.Num_materials_IGBT, 0, -1):
      if i == self.Num_materials_IGBT:
        num[i] = [self.materials_IGBT[i - 1].Rth]
        den[i] = [
          self.materials_IGBT[i - 1].Cth * self.materials_IGBT[i - 1].Rth, 1
        ]
      else:
        num[i] = [self.materials_IGBT[i - 1].Rth]
        den[i] = [
          self.materials_IGBT[i - 1].Cth * self.materials_IGBT[i - 1].Rth, 1
        ]
        num[i] = poly_somme(convolve(den[i + 1], num[i]),
                            convolve(den[i], num[i + 1]))
        den[i] = convolve(den[i + 1], den[i])

    self.num_foster_IGBT = num
    self.den_foster_IGBT = den

    num_diode = {}
    den_diode = {}

    # Calculating for Diode
    for i in range(self.Num_materials_diode, 0, -1):
      if i == self.Num_materials_diode:
        num_diode[i] = [self.materials_diode[i - 1].Rth]
        den_diode[i] = [
          self.materials_diode[i - 1].Cth * self.materials_diode[i - 1].Rth, 1
        ]
      else:
        num_diode[i] = [self.materials_diode[i - 1].Rth]
        den_diode[i] = [
          self.materials_diode[i - 1].Cth * self.materials_diode[i - 1].Rth, 1
        ]
        num_diode[i] = poly_somme(convolve(den_diode[i + 1], num_diode[i]),
                                  convolve(den_diode[i], num_diode[i + 1]))
        den_diode[i] = convolve(den_diode[i + 1], den_diode[i])

    self.num_foster_diode = num_diode
    self.den_foster_diode = den_diode


  def calculate_zth_cf(self):
    """
    Calculate the case-to-fluid thermal impedance.
    """

    self.num_hf = self.Rth_hf
    self.den_hf = 0
    if (self.Cth_ch==0):
      self.num_cf = poly_somme(convolve(self.num_hf, [1]), self.Rth_ch)
      self.den_cf = 1
    else:
      self.num_cf = poly_somme(convolve(self.num_hf, [self.Rth_ch * self.Cth_ch, 1]), self.Rth_ch)
      self.den_cf = [self.Rth_ch * self.Cth_ch, 1]



import numpy as np

def poly_somme(P, Q):
    """
    Function to add two polynomials P and Q represented as arrays or single coefficients.

    Parameters:
    P, Q (array or int or float): Polynomials or coefficients to be added. 

    Returns:
    R (array): Resulting polynomial after addition.
    """
    
    # If both P and Q are int or float, simply add them
    if isinstance(P, (int, float)) and isinstance(Q, (int, float)):
        return P + Q

    # Convert P and Q to numpy arrays if they are int or float
    if isinstance(P, (int, float)):
        P = np.array([P])
    elif isinstance(Q, (int, float)):
        Q = np.array([Q])

    Np = P.shape[0]  # Length of P
    Nq = Q.shape[0]  # Length of Q

    # If P is longer than Q, prepend Q with zeros to match P's length
    if Np > Nq:
        Q = np.concatenate((np.zeros(Np - Nq, dtype=P.dtype), Q))
    # If Q is longer than P, prepend P with zeros to match Q's length
    elif Np < Nq:
        P = np.concatenate((np.zeros(Nq - Np, dtype=Q.dtype), P))

    R = P + Q  # Add the two polynomials
    return R  # Return the resulting polynomial



import importlib.util

def import_variables(file_path):
    """
    Import the variables module from a Python file.

    Parameters:
    file_path (str): The path to the Python file.

    Returns:
    module: The imported variables module.
    """
    spec = importlib.util.spec_from_file_location("variables", file_path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module






from setuptools import setup, find_packages


setup(
    name='PowDeRUL',
    version='1.0.21',
    license='MIT',
    author='PGarn',
    author_email='paul.garnier@ens-rennes.fr',
    url='https://github.com/PGarn/LifeTime',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[
        'scipy>=1.10.1',
        'pandas>=2.0.1',
        'matplotlib>=3.7.1',
        'numpy>=1.24.1',
        'rainflow>=3.2.0',
        'openpyxl>=3.1.2',
    ],
)



class Materials:
    def __init__(self, Rth, Cth):
        """
        Initializes an instance of the Materials class.

        Parameters:
        Rth (float): Thermal resistance value
        Cth (float): Thermal capacitance value
        """
        self.Rth = Rth  # Assign thermal resistance
        self.Cth = Cth  # Assign thermal capacitance



import numpy as np

class Losses:
    def __init__(self, dic, file):
        """
        Initializes an instance of the Losses class.

        Parameters:
        dic (dict): Dictionary of constants and file parameters
        file (pandas DataFrame): A pandas DataFrame representation of the Excel file
        """
        # Extract relevant data and constants from the provided file and dictionary
        self.i_ref = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_current"]-1)].values
        self.cosphi = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_cosphi"]-1)].values
        self.m = file.iloc[(dic["excel_starting_row"]-1):(dic["excel_ending_row"]-1), (dic["excel_column_m"]-1)].values
      
        # Additional constants are also initialized here
        self.Vce0 = dic["Vce0"]
        self.Rce = dic["Rce"]
        self.f_sw = dic["f_sw"]
        self.E_sw = dic["E_sw"]
        self.Iref_sw = dic["Iref_sw"]
        self.Vdc = dic["Vdc"]
        self.Vref_sw = dic["Vref_sw"]
        self.kv_igbt = dic["kv_igbt"]
        self.Rd = dic["Rd"]
        self.E_d = dic["E_d"]
        self.Iref_d= dic["Iref_d"]
        self.ki_diode = dic["ki_diode"]
        self.Vref_d = dic["Vref_d"]
        self.Vd = dic["Vd"]
        self.kv_diode = dic["kv_diode"]
        self.number_IGBT = dic["number_IGBT"]
        self.number_IGBT_parallel = dic["number_IGBT_parallel"]

        # Initialize variables to hold losses
        self.P_T_cond = None
        self.P_T_sw = None
        self.pertes_igbt = None
        self.P_D_cond = None
        self.P_D_sw = None
        self.pertes_diode = None
        self.total_losses = None
        self.total_losses_cum = None
    
    def calculate_losses(self):
        """
        Calculates losses for both IGBT and Diode.

        IGBT losses are divided into conduction losses (P_T_cond) and switching losses (P_T_sw).
        Diode losses are divided into conduction losses (P_D_cond) and switching losses (P_D_sw).

        The total losses and their cumulative sum are also calculated and stored.
        """

        # Calculate IGBT losses
        self.P_T_cond = (1/(2*np.pi) + self.m*self.cosphi/8) * self.Vce0 * np.sqrt(2) * self.i_ref + (1/8 + self.m*self.cosphi/(3*np.pi)) * self.Rce * 2 * self.i_ref**2
        self.P_T_sw = self.f_sw * self.E_sw * np.sqrt(2) / np.pi * self.i_ref / self.Iref_sw * (self.Vdc / self.Vref_sw)**self.kv_igbt
        self.pertes_igbt = self.P_T_cond + self.P_T_sw

        # Calculate Diode losses
        self.P_D_cond = (1/(2*np.pi) + self.m*self.cosphi/8) * self.Vd * np.sqrt(2) * self.i_ref + (1/8 + self.m*self.cosphi/(3*np.pi)) * self.Rd * 2 * self.i_ref**2
        self.P_D_sw = self.f_sw * self.E_d * np.sqrt(2) / np.pi * (self.i_ref / self.Iref_d)**self.ki_diode * (self.Vdc / self.Vref_d)**self.kv_diode
        self.pertes_diode = self.P_D_cond + self.P_D_sw

        # Calculate total losses and their cumulative sum
        self.total_losses = np.asarray((self.pertes_igbt + self.pertes_diode) * self.number_IGBT, dtype=float)
        self.total_losses_cum = np.cumsum(self.total_losses)



import pandas as pd
# Importing required modules from the local directories
from powderul.function.counters import count
from powderul.function.cumul_endommagement import calculate_damage_cumulation
from powderul.function.losses import Losses
from powderul.function.temperature import calculate_temperature
from powderul.function.graphs import graph
from powderul.function.path_reader import import_variables

def global_calculation(variables_file_path):
  """
  This function executes a sequence of calculations related to thermal 
  and electrical properties of some materials, based on data from an Excel file.
  """
  # Choice of dictionary for the analysis. This could be modified in the future for flexibility.
  dic = import_variables(variables_file_path).variables
  
  # Load the Excel file defined in 'variables'
  xls = pd.ExcelFile(dic["excel_file_path"])
  
  # Read a specific sheet from the Excel file (also defined in 'variables')
  file = xls.parse(dic["excel_page"])

  
  # Instantiate a Losses object and calculate losses
  losses = Losses(dic, file)
  losses.calculate_losses()
  
  # Calculate the temperature based on the losses and thermal resistances
  Temp_IGBT, Temp_diode, Temp_case = calculate_temperature( losses, dic, file)

  # Calculate the counters and lifetimes for both IGBT and Diode
  counter_IGBT, counter_Nf_IGBT = count(Temp_IGBT, dic)
  counter_diode, counter_Nf_diode = count(Temp_diode, dic)

  # Display Torque, Speed, Current, Loss through a graph
  # Uncomment this line if you want to show the graphs
  # graph(file,dic,losses,Temp_IGBT,Temp_diode,Temp_case)
  
  # Calculate the cumulative damage law
  lifetime_IGBT, lifetime_diode, number_of_km_IGBT, e_kwh_byhours, efficiency = calculate_damage_cumulation(counter_IGBT, counter_Nf_IGBT, counter_diode, counter_Nf_diode, losses, dic, file)

  results=[["IGBT Cycles",lifetime_IGBT],["Diode Cycles",lifetime_diode],["Efficiency",efficiency],["e_kwh_byhours",e_kwh_byhours],["number_of_km_IGBT",number_of_km_IGBT]]
  
  # Return the calculated lifetimes and other datas
  return (results[0:3]) #add number_of_km_IGBT, e_kwh_byhours if you want

def global_graph(variables_file_path):
  """
  This function executes a sequence of calculations related to thermal 
  and electrical properties of some materials, based on data from an Excel file.
  """
  # Choice of dictionary for the analysis. This could be modified in the future for flexibility.
  dic = import_variables(variables_file_path).variables
  
  # Load the Excel file defined in 'variables'
  xls = pd.ExcelFile(dic["excel_file_path"])
  
  # Read a specific sheet from the Excel file (also defined in 'variables')
  file = xls.parse(dic["excel_page"])

  
  # Instantiate a Losses object and calculate losses
  losses = Losses(dic, file)
  losses.calculate_losses()
  
  # Calculate the temperature based on the losses and thermal resistances
  Temp_IGBT, Temp_diode, Temp_case = calculate_temperature( losses, dic, file)

  # Calculate the counters and lifetimes for both IGBT and Diode
  counter_IGBT, counter_Nf_IGBT = count(Temp_IGBT, dic)
  counter_diode, counter_Nf_diode = count(Temp_diode, dic)

  # Display Torque, Speed, Current, Loss through a graph
  graph(file,dic,losses,Temp_IGBT,Temp_diode,Temp_case)